@@ -31,12 +31,15 @@ struct PagePrivate
     /* Number of rows in this page or -1 if currently unknown */
     gint rows;
 
+    /* Color profile */
+    gchar *color_profile;
+
     /* Scanned image data */
     GdkPixbuf *image;
-  
+
     /* Page is getting data */
     gboolean scanning;
-  
+
     /* TRUE if have some page data */
     gboolean has_data;
 
@@ -45,7 +48,7 @@ struct PagePrivate
 
     /* Rotation of scanned data */
     Orientation orientation;
-    
+
     /* Crop */
     gboolean has_crop;
     gchar *crop_name;
@@ -62,6 +65,24 @@ page_new ()
 }
 
 
+void
+page_setup (Page *page, gint width, gint height, gint dpi, Orientation orientation)
+{ 
+    page->priv->orientation = orientation;
+    page->priv->dpi = dpi;
+    if (orientation == LEFT_TO_RIGHT || orientation == RIGHT_TO_LEFT)
+        page->priv->rows = width;
+    else
+        page->priv->rows = height;
+    page->priv->image = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE,
+                                        8,
+                                        width,
+                                        height);
+    g_return_if_fail (page->priv->image != NULL);
+    gdk_pixbuf_fill (page->priv->image, 0xFFFFFFFF);
+}
+
+
 void
 page_set_scan_area (Page *page, gint width, gint rows, gint dpi)
 {
@@ -88,10 +109,14 @@ page_set_scan_area (Page *page, gint width, gint rows, gint dpi)
 
     /* Create a white page */
     /* NOTE: Pixbuf only supports 8 bit RGB images */
+    if (page->priv->image)
+        g_object_unref (page->priv->image);
     page->priv->image = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE,
                                         8,
                                         width,
                                         height);
+    g_return_if_fail (page->priv->image != NULL);
+
     gdk_pixbuf_fill (page->priv->image, 0xFFFFFFFF);
     g_signal_emit (page, signals[SIZE_CHANGED], 0);
     g_signal_emit (page, signals[IMAGE_CHANGED], 0);
@@ -226,7 +251,7 @@ parse_line (Page *page, ScanLine *line, gint n, gboolean *size_changed)
                                 8, new_width, new_height);
 
         /* Copy old data */
-        gdk_pixbuf_fill (image, 0xFFFFFFFF);
+        gdk_pixbuf_fill (page->priv->image, 0xFFFFFFFF);
         if (page->priv->orientation == TOP_TO_BOTTOM || page->priv->orientation == LEFT_TO_RIGHT)
             gdk_pixbuf_copy_area (page->priv->image, 0, 0, width, height,
                                   image, 0, 0);
@@ -364,6 +389,7 @@ page_set_orientation (Page *page, Orientation orientation)
     gint left_steps, t;
     GdkPixbuf *image;
     gboolean size_changed = FALSE;
+    gint width, height;
 
     g_return_if_fail (page != NULL);
 
@@ -375,16 +401,17 @@ page_set_orientation (Page *page, Orientation orientation)
     if (left_steps < 0)
         left_steps += 4;
   
+    width = page_get_width (page);
+    height = page_get_height (page);
+  
     /* Rotate image */
     if (left_steps == 1)
         image = gdk_pixbuf_rotate_simple (page->priv->image, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
     else if (left_steps == 2)
         image = gdk_pixbuf_rotate_simple (page->priv->image, GDK_PIXBUF_ROTATE_UPSIDEDOWN);
-    else if (left_steps == 3)
-        image = gdk_pixbuf_rotate_simple (page->priv->image, GDK_PIXBUF_ROTATE_CLOCKWISE);
     else
-        image = gdk_pixbuf_rotate_simple (page->priv->image, GDK_PIXBUF_ROTATE_NONE);
-    gdk_pixbuf_unref (page->priv->image);
+        image = gdk_pixbuf_rotate_simple (page->priv->image, GDK_PIXBUF_ROTATE_CLOCKWISE);
+    g_object_unref (page->priv->image);
     page->priv->image = image;
     if (left_steps != 2)
         size_changed = TRUE;
@@ -396,21 +423,21 @@ page_set_orientation (Page *page, Orientation orientation)
         case 1:
             t = page->priv->crop_x;
             page->priv->crop_x = page->priv->crop_y;
-            page->priv->crop_y = page_get_width (page) - (t + page->priv->crop_width);
+            page->priv->crop_y = width - (t + page->priv->crop_width);
             t = page->priv->crop_width;
             page->priv->crop_width = page->priv->crop_height;
             page->priv->crop_height = t;
             break;
         /* 180 degrees */
         case 2:
-            page->priv->crop_x = page_get_width (page) - (page->priv->crop_x + page->priv->crop_width);
-            page->priv->crop_y = page_get_width (page) - (page->priv->crop_y + page->priv->crop_height);
+            page->priv->crop_x = width - (page->priv->crop_x + page->priv->crop_width);
+            page->priv->crop_y = width - (page->priv->crop_y + page->priv->crop_height);
             break;
         /* 90 degrees clockwise */
         case 3:
             t = page->priv->crop_y;
             page->priv->crop_y = page->priv->crop_x;
-            page->priv->crop_x = page_get_height (page) - (t + page->priv->crop_height);
+            page->priv->crop_x = height - (t + page->priv->crop_height);
             t = page->priv->crop_width;
             page->priv->crop_width = page->priv->crop_height;
             page->priv->crop_height = t;
@@ -420,7 +447,7 @@ page_set_orientation (Page *page, Orientation orientation)
 
     page->priv->orientation = orientation;
     if (size_changed)
-        g_signal_emit (page, signals[SIZE_CHANGED], 0);  
+        g_signal_emit (page, signals[SIZE_CHANGED], 0);
     g_signal_emit (page, signals[IMAGE_CHANGED], 0);
     g_signal_emit (page, signals[ORIENTATION_CHANGED], 0);
     g_signal_emit (page, signals[CROP_CHANGED], 0);
@@ -510,7 +537,19 @@ page_get_scan_height (Page *page)
         return gdk_pixbuf_get_height (page->priv->image);
     else
         return gdk_pixbuf_get_width (page->priv->image);  
+}
 
+
+void page_set_color_profile (Page *page, const gchar *color_profile)
+{
+     g_free (page->priv->color_profile);
+     page->priv->color_profile = g_strdup (color_profile);
+}
+
+
+const gchar *page_get_color_profile (Page *page)
+{
+     return page->priv->color_profile;
 }
 
 
@@ -718,56 +757,40 @@ GdkPixbuf *
 page_get_image (Page *page)
 {
     g_return_val_if_fail (page != NULL, NULL);
-    return gdk_pixbuf_copy (page->priv->image);
+    return g_object_ref (page->priv->image);
 }
 
 
 GdkPixbuf *
 page_get_cropped_image (Page *page)
 {
-    GdkPixbuf *image, *cropped_image, *i;
+    GdkPixbuf *image, *cropped_image;
     gint x, y, w, h, pw, ph;
 
     g_return_val_if_fail (page != NULL, NULL);
-    
+
     image = page_get_image (page);
-    
+
     if (!page->priv->has_crop)
         return image;
-    
+
     x = page->priv->crop_x;
     y = page->priv->crop_y;
     w = page->priv->crop_width;
     h = page->priv->crop_height;
     pw = gdk_pixbuf_get_width (image);
     ph = gdk_pixbuf_get_height (image);
-    
+
     /* Trim crop */
     if (x + w >= pw)
         w = pw - x;
     if (y + h >= ph)
         h = ph - y;
-    
+
     cropped_image = gdk_pixbuf_new_subpixbuf (image, x, y, w, h);
     g_object_unref (image);
-    
-    i = gdk_pixbuf_copy (cropped_image);
-    g_object_unref (cropped_image);
 
-    return i;
-}
-
-
-static GFileOutputStream *
-open_file (const gchar *uri, GError **error)
-{
-    GFile *file;
-    GFileOutputStream *stream; 
-
-    file = g_file_new_for_uri (uri);
-    stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
-    g_object_unref (file);
-    return stream;
+    return cropped_image;
 }
 
 
@@ -778,70 +801,78 @@ write_pixbuf_data (const gchar *buf, gsize count, GError **error, GFileOutputStr
 }
 
 
-gboolean
-page_save_jpeg (Page *page, const gchar *uri, GError **error)
+static gchar *
+get_icc_data_encoded (const gchar *icc_profile_filename)
 {
-    GdkPixbuf *image;
-    GFileOutputStream *stream;
-    gboolean result;
+    gchar *contents = NULL;
+    gchar *contents_encode = NULL;
+    gsize length;
+    gboolean ret;
+    GError *error = NULL;
+
+    /* Get binary data */
+    ret = g_file_get_contents (icc_profile_filename, &contents, &length, &error);
+    if (!ret) {
+        g_warning ("failed to get icc profile data: %s", error->message);
+        g_error_free (error);
+    }
+    else {
+        /* Encode into base64 */
+        contents_encode = g_base64_encode ((const guchar *) contents, length);
+    }
   
-    stream = open_file (uri, error);
-    if (!stream)
-        return FALSE;
-
-    image = page_get_cropped_image (page);
-    result = gdk_pixbuf_save_to_callback (image,
-                                          (GdkPixbufSaveFunc) write_pixbuf_data, stream,
-                                          "jpeg", error,
-                                          "quality", "90",
-                                          NULL);
-    g_object_unref (image);
-    g_object_unref (stream);
-
-    return result;
+    g_free (contents);
+    return contents_encode;
 }
 
 
 gboolean
-page_save_png (Page *page, const gchar *uri, GError **error)
+page_save (Page *page, const gchar *type, GFile *file, GError **error)
 {
-    GdkPixbuf *image;
     GFileOutputStream *stream;
-    gboolean result;
-  
-    stream = open_file (uri, error);
+    GdkPixbuf *image;
+    gboolean result = FALSE;
+    gchar *icc_profile_data = NULL;
+
+    stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
     if (!stream)
         return FALSE;
 
     image = page_get_cropped_image (page);
-    result = gdk_pixbuf_save_to_callback (image,
-                                          (GdkPixbufSaveFunc) write_pixbuf_data, stream,
-                                          "png", error,
-                                          NULL);
-    g_object_unref (image);
-    g_object_unref (stream);
 
-    return result;
-}
+    if (page->priv->color_profile != NULL)
+        icc_profile_data = get_icc_data_encoded (page->priv->color_profile);
 
+    if (strcmp (type, "jpeg") == 0) {
+        /* ICC profile is awaiting review in gtk2+ bugzilla */
+        gchar *keys[] = { "quality", /* "icc-profile", */ NULL };
+        gchar *values[] = { "90", /* icc_profile_data, */ NULL };
+        result = gdk_pixbuf_save_to_callbackv (image,
+                                               (GdkPixbufSaveFunc) write_pixbuf_data, stream,
+                                               "jpeg", keys, values, error);
+    }
+    else if (strcmp (type, "png") == 0) {
+        gchar *keys[] = { "icc-profile", NULL };
+        gchar *values[] = { icc_profile_data, NULL };
+        if (icc_profile_data == NULL)
+            keys[0] = NULL;
+        result = gdk_pixbuf_save_to_callbackv (image,
+                                               (GdkPixbufSaveFunc) write_pixbuf_data, stream,
+                                               "png", keys, values, error);
+    }
+    else if (strcmp (type, "tiff") == 0) {
+        gchar *keys[] = { "compression", "icc-profile", NULL };
+        gchar *values[] = { "8" /* Deflate compression */, icc_profile_data, NULL };
+        if (icc_profile_data == NULL)
+            keys[1] = NULL;
+        result = gdk_pixbuf_save_to_callbackv (image,
+                                               (GdkPixbufSaveFunc) write_pixbuf_data, stream,
+                                               "tiff", keys, values, error);
+    }
+    else
+        result = FALSE; // FIXME: Set GError
 
-gboolean
-page_save_tiff (Page *page, const gchar *uri, GError **error)
-{
-    GdkPixbuf *image;
-    GFileOutputStream *stream;
-    gboolean result;
-  
-    stream = open_file (uri, error);
-    if (!stream)
-        return FALSE;
-
-    image = page_get_cropped_image (page);
-    result = gdk_pixbuf_save_to_callback (image,
-                                          (GdkPixbufSaveFunc) write_pixbuf_data, stream,
-                                          "tiff", error,
-                                          "compression", "8", /* Deflate compression */
-                                          NULL);
+    g_free (icc_profile_data);
     g_object_unref (image);
     g_object_unref (stream);
 
@@ -853,7 +884,8 @@ static void
 page_finalize (GObject *object)
 {
     Page *page = PAGE (object);
-    g_object_unref (page->priv->image);
+    if (page->priv->image)
+        g_object_unref (page->priv->image);
     page->priv->image = NULL;
     G_OBJECT_CLASS (page_parent_class)->finalize (object);
 }
